//======================================================================
// Copyright (C) 2002 Daniel Heck
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//  
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//  
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
//======================================================================
#ifndef PX_TOOLS_HH
#define PX_TOOLS_HH

#include <string>

namespace px
{
    class Nocopy {
        Nocopy(const Nocopy&);
        Nocopy& operator=(const Nocopy&);
      public:
        Nocopy() {}
    };
}

namespace px
{
// This handle class comes (almost verbatim) from TC++PL.
    template <class X> class Handle {
        X *rep;
        int *cnt;
    public:
        X *operator->() { return rep; }
        X *get() const { return rep; }
        X &operator *() { return *rep; }

        explicit Handle(X *p=0) : rep(p), cnt(new int(1)) {}
        Handle(const Handle<X>& a) : rep(a.rep), cnt(a.cnt) {
            ++(*cnt);
        }
        Handle<X> &operator=(const Handle<X> &a) {
            if (cnt == a.cnt) return *this;
            if (--(*cnt) == 0) {
                delete rep;
                delete cnt;
            }
            rep = a.rep;
            cnt = a.cnt;
            (*cnt)++;
            return *this;
        }
        ~Handle() {
            if (--(*cnt) == 0) {
                delete rep;
                delete cnt;
            }
        }
    };

//     template <>
//     class hash<string> {
//         hash<const char *> hasher;
//     public:
//         size_t operator() (const string &s) const {
//             return hasher(s.c_str());
//         }
//     };

    template <class Iter> 
    Iter next(const Iter i) {
        Iter j=i;
        return ++j;
    }
    template <class Iter> 
    Iter prev(const Iter i) {
        Iter j=i;
        return --j;
    }


    template <class Exc> void
    Assert(bool cond) 
    {
        if (!cond)
            throw Exc();
    }

    template <class For> void
    delete_sequence(For begin, For end) 
    {
        while (begin != end) 
            delete *begin++;
    }

    template <class For> void 
    delete_map(For begin, For end) 
    {
        while (begin != end) 
        {
            delete begin->second;
            ++begin;
        }
    }

    template <class T> const T &
    Max(const T &a, const T &b)
    {
        return (a > b) ? a : b;
    }

    template <class T> const T &
    Min(const T &a, const T &b)
    {
        return (a < b) ? a : b;
    }


    template <class OutIt>
    void split_copy(const std::string &str, char sep, OutIt out)
    {
        std::string::const_iterator i,j;
        i=str.begin();
        while (true) {
            j = std::find(i, str.end(), sep);
            *out = std::string(i,j);
            ++out;
            if (j==str.end())
                break;
            i=j+1;
        }
    }
}

#endif
